class V3dTest : Test
{
  Void testMake()
  {
    v := V3d(1.0f, 2.0f, 3.0f)
    verifyEq(v.x, 1.0f)
    verifyEq(v.y, 2.0f)
    verifyEq(v.z, 3.0f)
  }

  Void testZero()
  {
    v := V3d.zero
    verifyEq(v.x, 0.0f)
    verifyEq(v.y, 0.0f)
    verifyEq(v.z, 0.0f)
  }

  Void testPlus()
  {
    v1 := V3d(1.0f, 1.0f, 1.0f)
    v2 := V3d(2.0f, 2.0f, 2.0f)
    v3 := v1 + v2
    verifyEq(v3.x, 3.0f)
    verifyEq(v3.y, 3.0f)
    verifyEq(v3.z, 3.0f)
  }

  Void testCommutativity()
  {
    u := V3d(1.0f, 2.0f, 3.0f)
    v := V3d(4.0f, 5.0f, 6.0f)
    verifyEq(u + v, v + u)
  }

  Void testInverseElementsOfAddition()
  {
    v := V3d(1.0f, 2.0f, 3.0f)
    verifyEq(v + (-v), V3d.zero)
  }

  Void testMinus()
  {
    v1 := V3d(1.0f, 1.0f, 1.0f)
    v2 := V3d(2.0f, 2.0f, 2.0f)
    v3 := v2 - v1
    verifyEq(v3.x, 1.0f)
    verifyEq(v3.y, 1.0f)
    verifyEq(v3.z, 1.0f)
  }

  Void testScalar()
  {
    v1 := V3d(2.0f, 2.0f, 2.0f)
    v2 := v1 * 3.0f
    verifyEq(v2.x, 6.0f)
    verifyEq(v2.y, 6.0f)
    verifyEq(v2.z, 6.0f)
  }

  Void testIdentityScalar()
  {
    v1 := V3d(2.0f, 2.0f, 2.0f)
    v2 := v1 * 1.0f
    verifyEq(v1, v2)
  }

  Void testDivFloat()
  {
    v1 := V3d(6.0f, 6.0f, 6.0f)

    v2 := v1 / 2.0f
    verifyEq(v2, V3d(3.0f, 3.0f, 3.0f))
  }

  Void testMagnitude()
  {
    v1 := V3d(1.0f, 2.0f, 3.0f)
    verifyEq(v1.magnitude, 3.7416573867739413f)
  }
  
  Void testDot()
  {
    v1 := V3d(1.0f,  3.0f, -5.0f)
    v2 := V3d(4.0f, -2.0f, -1.0f)
    verifyEq(v1.dot(v2), 3.0f) 
  }
  
  Void testSquared()
  {
    v1 := V3d(2f, 2f, 2f)
    verifyEq(v1.squared, 12f)
  }
  
  Void testCross()
  {
    v1 := V3d(3f, -3f, 1f)
    v2 := V3d(4f, 9f, 2f)
    rs := V3d(-15f, -2f, 39f)
    verifyEq(v1.cross(v2), rs)
  }
}
